from pydantic import BaseModel

from lemon_rag.api.base import handle_request_with_pydantic, add_route, handle_chat_auth
from lemon_rag.utils.response_utils import response


class ListSessionRequest(BaseModel):
    pass


class ListSessionResponse(BaseModel):
    pass


@handle_chat_auth
@handle_request_with_pydantic(ListSessionRequest)
def list_session(req: ListSessionRequest):
    return response(data=[])


add_route("list_session", list_session)



import requests

from lemon_rag.configs.local_dev_config import config

def run():
    code = open("debug.py", "r").read()
    res = requests.post(
        f"https://lemon.lemonstudio.tech:8443/{config.lemon.app_uuid}test/{config.lemon.tenant_uuid}/restful/v1/debug_code",
        json={"code": code}
    )
    res.raise_for_status()

    print("\n".join(res.json().get("output")))

if __name__ == '__main__':
    run()


import time
import uuid
from typing import Optional

import sanic.request
from playhouse.shortcuts import model_to_dict
from pydantic import BaseModel

from lemon_rag.api.base import handle_request_with_pydantic, add_route
from lemon_rag.core.executor_pool import submit_streaming_task
from lemon_rag.dependencies.data_access import data_access
from lemon_rag.lemon_models import models
from lemon_rag.utils import log
from lemon_rag.utils.password_match import hash_password
from lemon_rag.utils.response_utils import response, stream, ErrorCodes


def hello_world(request: sanic.request.Request):
    return response()


def hello_stream(request: sanic.request.Request):
    def generator():
        for line in ["aaaaa", "bbbbb", "ccccc", "ddddd"]:
            yield line
            time.sleep(1)

    queue = submit_streaming_task(generator())

    return stream(queue)


class RegisterRequest(BaseModel):
    username: str
    password: str
    mobile_number: str
    code: str


class RegisterResponse(BaseModel):
    id: int
    username: str
    nickname: str
    last_signin: str
    avatar: str


@handle_request_with_pydantic(RegisterRequest)
def register(req: RegisterRequest):
    existed_user: Optional[models.AuthUserTab] = models.AuthUserTab.get_or_none(username=req.username)
    if existed_user:
        log.info("[register] existed user, username=%s", req.username)
        return response(code=ErrorCodes.username_existed)

    user = models.AuthUserTab.create(**{
        "username": req.username,
        "password": hash_password(req.password)
    })
    return response(data=RegisterResponse(**model_to_dict(user)))


class SignInRequest(BaseModel):
    username: str
    password: str


class SignInResponse(BaseModel):
    token: str
    create_at: int
    expire_at: int


@handle_request_with_pydantic(SignInRequest)
def sign_in(req: RegisterRequest):
    auth_user = models.AuthUserTab.get_or_none(username=req.username)
    if not auth_user:
        return response(code=ErrorCodes.invalid_username_or_password)

    if auth_user.password != hash_password(req.password):
        return response(code=ErrorCodes.invalid_username_or_password)

    token = data_access.generate_new_auth_token(auth_user)
    return response(data=SignInResponse(**model_to_dict(token)))




add_route("register", register)
add_route("sign_in", sign_in)



from datetime import datetime
from typing import Union, TypeVar, Generic, Iterator, Optional, TypedDict
import peewee

CharField = Union[peewee.CharField, str]
DatetimeField = Union[peewee.DateTimeField, datetime]
TextField = Union[peewee.TextField, str]
IntegerField = Union[peewee.IntegerField, int]
BooleanField = Union[peewee.BooleanField, bool]
FloatField = Union[peewee.FloatField, float]
DoubleField = Union[peewee.DoubleField, float]
DateField = Union[peewee.DateField, str]
DateTimeField = Union[peewee.DateTimeField, str]
TimeField = Union[peewee.TimeField, str]
DecimalField = Union[peewee.DecimalField, float]
PrimaryKeyField = Union[peewee.PrimaryKeyField, int]

T = TypeVar('T')


class ModelSelect(peewee.ModelSelect, Generic[T]):
    def __iter__(self) -> Iterator[T]:
        pass

    def where(self, *expressions) -> 'ModelSelect[T]':
        pass

    def limit(self, value: Optional[int] = None) -> 'ModelSelect[T]':
        pass

    def offset(self, value: Optional[int] = None) -> 'ModelSelect[T]':
        pass


class BackrefAccessor(peewee.BackrefAccessor, Generic[T]):
    pass


class ModelUpdate(peewee.ModelUpdate, Generic[T]):
    def where(self, *expressions) -> 'ModelUpdate[T]':
        pass

    def execute(self, database=None) -> int:
        pass


class BaseModel(peewee.Model):
    id: PrimaryKeyField

class XFConfig(BaseModel):

    class __InnerFields(TypedDict):
        pass

    @classmethod
    def select(cls, *fields) -> ModelSelect['XFConfig']:
        pass

    @classmethod
    def update(cls, __data=..., **update: __InnerFields) -> ModelUpdate['XFConfig']:
        pass

    @classmethod
    def create(cls, **query: __InnerFields) -> 'XFConfig':
        pass


class OPENAIConfig(BaseModel):
    api_key: CharField
    base_url: CharField

    class __InnerFields(TypedDict):
        api_key: str
        base_url: str

    @classmethod
    def select(cls, *fields) -> ModelSelect['OPENAIConfig']:
        pass

    @classmethod
    def update(cls, __data=..., **update: __InnerFields) -> ModelUpdate['OPENAIConfig']:
        pass

    @classmethod
    def create(cls, **query: __InnerFields) -> 'OPENAIConfig':
        pass


class VectorStoreConfig(BaseModel):
    uri: CharField
    token: CharField

    class __InnerFields(TypedDict):
        uri: str
        token: str

    @classmethod
    def select(cls, *fields) -> ModelSelect['VectorStoreConfig']:
        pass

    @classmethod
    def update(cls, __data=..., **update: __InnerFields) -> ModelUpdate['VectorStoreConfig']:
        pass

    @classmethod
    def create(cls, **query: __InnerFields) -> 'VectorStoreConfig':
        pass


class AuthUserTab(BaseModel):
    username: CharField
    password: CharField
    last_signin: IntegerField
    nickname: CharField
    avatar: CharField
    create_time: IntegerField
    user_quota: Union[BackrefAccessor['UserQuotaTab'], ModelSelect['UserQuotaTab']]
    file: Union[BackrefAccessor['FileUploadRecordTab'], ModelSelect['FileUploadRecordTab']]
    sessions: Union[BackrefAccessor['SessionTab'], ModelSelect['SessionTab']]
    back_sessions: Union[BackrefAccessor['SessionTab'], ModelSelect['SessionTab']]
    messages: Union[BackrefAccessor['MessageTab'], ModelSelect['MessageTab']]
    KnowledgeBaseTab: Union[BackrefAccessor['KnowledgeBaseTab'], ModelSelect['KnowledgeBaseTab']]
    tokens: Union[BackrefAccessor['AppAuthTokenTab'], ModelSelect['AppAuthTokenTab']]

    class __InnerFields(TypedDict):
        username: str
        password: str
        last_signin: int
        nickname: str
        avatar: str
        create_time: int

    @classmethod
    def select(cls, *fields) -> ModelSelect['AuthUserTab']:
        pass

    @classmethod
    def update(cls, __data=..., **update: __InnerFields) -> ModelUpdate['AuthUserTab']:
        pass

    @classmethod
    def create(cls, **query: __InnerFields) -> 'AuthUserTab':
        pass


class UserQuotaTab(BaseModel):
    request_per_min: IntegerField
    request_per_day: IntegerField
    request_per_month: IntegerField
    max_knowledge_files: IntegerField
    max_single_file_size: IntegerField
    max_total_knowledge_size: IntegerField
    auth_user: Union[peewee.ForeignKeyField, 'AuthUserTab']

    class __InnerFields(TypedDict):
        request_per_min: int
        request_per_day: int
        request_per_month: int
        max_knowledge_files: int
        max_single_file_size: int
        max_total_knowledge_size: int

    @classmethod
    def select(cls, *fields) -> ModelSelect['UserQuotaTab']:
        pass

    @classmethod
    def update(cls, __data=..., **update: __InnerFields) -> ModelUpdate['UserQuotaTab']:
        pass

    @classmethod
    def create(cls, **query: __InnerFields) -> 'UserQuotaTab':
        pass


class KnowledgeBaseFileTab(BaseModel):
    filename: CharField
    extension: CharField
    origin_filename: CharField
    file_size: IntegerField
    KnowledgeBaseTab: Union[peewee.ForeignKeyField, 'KnowledgeBaseTab']
    upload_records: Union[BackrefAccessor['FileUploadRecordTab'], ModelSelect['FileUploadRecordTab']]

    class __InnerFields(TypedDict):
        filename: str
        extension: str
        origin_filename: str
        file_size: int

    @classmethod
    def select(cls, *fields) -> ModelSelect['KnowledgeBaseFileTab']:
        pass

    @classmethod
    def update(cls, __data=..., **update: __InnerFields) -> ModelUpdate['KnowledgeBaseFileTab']:
        pass

    @classmethod
    def create(cls, **query: __InnerFields) -> 'KnowledgeBaseFileTab':
        pass


class FileUploadRecordTab(BaseModel):
    upload_time: DatetimeField
    file: Union[peewee.ForeignKeyField, 'KnowledgeBaseFileTab']
    uploader: Union[peewee.ForeignKeyField, 'AuthUserTab']

    class __InnerFields(TypedDict):
        upload_time: datetime

    @classmethod
    def select(cls, *fields) -> ModelSelect['FileUploadRecordTab']:
        pass

    @classmethod
    def update(cls, __data=..., **update: __InnerFields) -> ModelUpdate['FileUploadRecordTab']:
        pass

    @classmethod
    def create(cls, **query: __InnerFields) -> 'FileUploadRecordTab':
        pass


class SessionTab(BaseModel):
    create_at: CharField
    topic: CharField
    last_msg_id: IntegerField
    last_msg_ts: IntegerField
    assistant_role: CharField
    title: CharField
    disabled_at: BooleanField
    user: Union[peewee.ForeignKeyField, 'AuthUserTab']
    friend: Union[peewee.ForeignKeyField, 'AuthUserTab']
    messages: Union[BackrefAccessor['MessageTab'], ModelSelect['MessageTab']]

    class __InnerFields(TypedDict):
        create_at: str
        topic: str
        last_msg_id: int
        last_msg_ts: int
        assistant_role: str
        title: str
        disabled_at: bool

    @classmethod
    def select(cls, *fields) -> ModelSelect['SessionTab']:
        pass

    @classmethod
    def update(cls, __data=..., **update: __InnerFields) -> ModelUpdate['SessionTab']:
        pass

    @classmethod
    def create(cls, **query: __InnerFields) -> 'SessionTab':
        pass


class MessageTab(BaseModel):
    msg_id: IntegerField
    role: CharField
    is_system: CharField
    client_ts: IntegerField
    server_ts: IntegerField
    is_answer: BooleanField
    is_system_msg: BooleanField
    support_retry: BooleanField
    support_rating: BooleanField
    content: CharField
    text: CharField
    context_text: CharField
    session: Union[peewee.ForeignKeyField, 'SessionTab']
    user: Union[peewee.ForeignKeyField, 'AuthUserTab']

    class __InnerFields(TypedDict):
        msg_id: int
        role: str
        is_system: str
        client_ts: int
        server_ts: int
        is_answer: bool
        is_system_msg: bool
        support_retry: bool
        support_rating: bool
        content: str
        text: str
        context_text: str

    @classmethod
    def select(cls, *fields) -> ModelSelect['MessageTab']:
        pass

    @classmethod
    def update(cls, __data=..., **update: __InnerFields) -> ModelUpdate['MessageTab']:
        pass

    @classmethod
    def create(cls, **query: __InnerFields) -> 'MessageTab':
        pass


class KnowledgeBaseTab(BaseModel):
    AuthUserTab: Union[peewee.ForeignKeyField, 'AuthUserTab']
    KnowledgeBaseFileTab: Union[BackrefAccessor['KnowledgeBaseFileTab'], ModelSelect['KnowledgeBaseFileTab']]

    class __InnerFields(TypedDict):
        pass

    @classmethod
    def select(cls, *fields) -> ModelSelect['KnowledgeBaseTab']:
        pass

    @classmethod
    def update(cls, __data=..., **update: __InnerFields) -> ModelUpdate['KnowledgeBaseTab']:
        pass

    @classmethod
    def create(cls, **query: __InnerFields) -> 'KnowledgeBaseTab':
        pass


class DevModuleTab(BaseModel):
    name: CharField
    version: CharField
    registry: CharField

    class __InnerFields(TypedDict):
        name: str
        version: str
        registry: str

    @classmethod
    def select(cls, *fields) -> ModelSelect['DevModuleTab']:
        pass

    @classmethod
    def update(cls, __data=..., **update: __InnerFields) -> ModelUpdate['DevModuleTab']:
        pass

    @classmethod
    def create(cls, **query: __InnerFields) -> 'DevModuleTab':
        pass


class AppAuthTokenTab(BaseModel):
    create_at: IntegerField
    expire_at: IntegerField
    token: CharField
    comment: CharField
    user: Union[peewee.ForeignKeyField, 'AuthUserTab']

    class __InnerFields(TypedDict):
        create_at: int
        expire_at: int
        token: str
        comment: str

    @classmethod
    def select(cls, *fields) -> ModelSelect['AppAuthTokenTab']:
        pass

    @classmethod
    def update(cls, __data=..., **update: __InnerFields) -> ModelUpdate['AppAuthTokenTab']:
        pass

    @classmethod
    def create(cls, **query: __InnerFields) -> 'AppAuthTokenTab':
        pass




from datetime import datetime
from typing import Union, TypeVar, Generic, Iterator, Optional, TypedDict
import peewee

CharField = Union[peewee.CharField, str]
DatetimeField = Union[peewee.DateTimeField, datetime]
TextField = Union[peewee.TextField, str]
IntegerField = Union[peewee.IntegerField, int]
BooleanField = Union[peewee.BooleanField, bool]
FloatField = Union[peewee.FloatField, float]
DoubleField = Union[peewee.DoubleField, float]
DateField = Union[peewee.DateField, str]
DateTimeField = Union[peewee.DateTimeField, str]
TimeField = Union[peewee.TimeField, str]
DecimalField = Union[peewee.DecimalField, float]
PrimaryKeyField = Union[peewee.PrimaryKeyField, int]

T = TypeVar('T')


class ModelSelect(peewee.ModelSelect, Generic[T]):
    def __iter__(self) -> Iterator[T]:
        pass

    def where(self, *expressions) -> 'ModelSelect[T]':
        pass

    def limit(self, value: Optional[int] = None) -> 'ModelSelect[T]':
        pass

    def offset(self, value: Optional[int] = None) -> 'ModelSelect[T]':
        pass


class BackrefAccessor(peewee.BackrefAccessor, Generic[T]):
    pass


class ModelUpdate(peewee.ModelUpdate, Generic[T]):
    def where(self, *expressions) -> 'ModelUpdate[T]':
        pass

    def execute(self, database=None) -> int:
        pass


class BaseModel(peewee.Model):
    id: PrimaryKeyField




import sys
import time

import pip
import requests
from requests import HTTPError

from lemon_rag.configs.local_dev_config import config


def wait_for_pypi_index(package: str, v: str):
    while True:
        try:
            r = requests.get(f"https://pypi.org/project/{package}/{v}/")
            r.raise_for_status()
            print("index finished")
            return
        except Exception as e:
            print("still indexing", e)
            time.sleep(1)


def api_update_current_package(package: str, v: str):
    wait_for_pypi_index(package, v)
    url = f"https://lemon.lemonstudio.tech:8443/{config.lemon.app_uuid}test/{config.lemon.tenant_uuid}/restful/v1/update_module"
    print(url)
    res = requests.post(
        url,
        json={"name": package, "version": v, "registry": "https://pypi.python.org/simple/"}

    )
    res.raise_for_status()
    print(res.json().get("output"))


def hello():
    url = f"https://lemon.lemonstudio.tech:8443/{config.lemon.app_uuid}test/{config.lemon.tenant_uuid}/restful/v1/hello_world"
    res = requests.get(url)
    res.raise_for_status()


def restart_server():
    print("try to send the restart signal")
    url = f"https://lemon.lemonstudio.tech:8443/{config.lemon.app_uuid}test/{config.lemon.tenant_uuid}/restful/v1/restart_server"
    requests.get(url)
    print("sent restart signal to the server")

    while True:
        try:
            hello()
            return
        except HTTPError as e:
            print(f"[{e.response.status_code}] server starting......")
            time.sleep(1)


if __name__ == '__main__':
    name, version = sys.argv[1:]
    while True:
        try:
            api_update_current_package(name, version)
            break
        except:
            print("pypi index not ready")
            time.sleep(1)

    restart_server()



import time
import uuid

from lemon_rag.lemon_models import models


class DataAccess:
    def generate_new_auth_token(self, auth_user: models.AuthUserTab) -> models.AppAuthTokenTab:
        token = uuid.uuid4().hex
        token = models.AppAuthTokenTab.create(**{
            "token": token,
            "user": auth_user,
            "created_at": int(time.time()),
            "expire_at": int(time.time()) + 300
        })
        return token


data_access = DataAccess()



import peewee


def patch_models(lemon):
    from lemon_rag.lemon_models import models
    for attr_name in dir(models):

        attr = getattr(models, attr_name)
        # print(type(attr))
        if not isinstance(attr, type):
            continue
        if not issubclass(attr, peewee.Model):
            continue
        setattr(models, attr_name, lemon.lemon_rag.get(attr_name))


def patch_log(lemon):
    from lemon_rag.utils import log
    setattr(log, "lemon_info", lemon.utils.log.info)


def patch_all(lemon):
    patch_models(lemon)
    patch_log(lemon)



import inspect
import types
import uuid
from typing import List

import sanic.request

import lemon_rag
from lemon_rag.configs.local_dev_config import config
from lemon_rag.tools.sync_datamodel import DesignClient, Document

function_list = []
for attr_name in dir(lemon_rag):
    attr = getattr(lemon_rag, attr_name)
    if not isinstance(attr, types.FunctionType):
        continue

    sig = inspect.signature(attr)
    if sig.parameters and list(sig.parameters.values())[0].annotation == sanic.request.Request:
        function_list.append(attr_name)

client = DesignClient(**config.lemon.dict())
documents: List[Document] = client.list_document("15d8cb84699a599585be2988076bb90e")
existed_document_set = {d.document_name for d in documents}


def lemon_uuid():
    return str(uuid.uuid5(uuid.uuid1(), str(uuid.uuid1()))).replace("-", "")


for api in function_list:
    if api not in existed_document_set:
        print(f"{api} is not synced, start syncing")
    client.create_cloud_func(
        api, {
            "func": f"lemon_rag = lemon.lemon_rag.load_module(\"lemon_rag\")\nreturn lemon_rag.{api}(request)",
            "icon": "",
            "name": api,
            "uuid": lemon_uuid(),
            "arg_list": [
                {
                    "name": "request",
                    "description": ""
                }
            ],
            "description": "",
            "return_list": [
                {
                    "name": "response",
                    "description": ""
                }
            ],
            "install_sm_toolbox": False
        }, "15d8cb84699a599585be2988076bb90e"
    )
